@@ -50,7 +50,7 @@ import nu.xom.Text;
  * </p>
  * 
  * @author Elliotte Rusty Harold
- * @version 1.0
+ * @version 1.2.3
  *
  */
 public class ElementTest extends XOMTestCase {
@@ -119,6 +119,26 @@ public class ElementTest extends XOMTestCase {
         
     }
     
+    private static class TestElement extends Element {
+
+        public TestElement(String name) {
+            super(name);
+        }
+        
+        public String toString() {
+            return "foo";
+        }
+        
+    }
+    
+    
+    public void testOverrideToString() {
+        
+        Element element = new TestElement("bar");
+        assertEquals("foo", element.toString());
+        
+    }
+    
     
     public void testGetFirstChildElement() {
         
@@ -137,11 +157,37 @@ public class ElementTest extends XOMTestCase {
         );
         
     }
+    
+    
+    public void testElementNamedXMLNS() {
+        
+        String name = "xmlns";
+        Element e = new Element(name);
+        
+        assertEquals(name, e.getLocalName());
+        assertEquals(name, e.getQualifiedName());
+        assertEquals("",   e.getNamespacePrefix());
+        assertEquals("",   e.getNamespaceURI());
+        
+    }
+
+    
+    public void testElementWithPrefixXMLNS() {
+        
+        try {
+            new Element("xmlns:foo", "http://www.example.org/");
+            fail("Allowed xmlns prefix on element");
+        }
+        catch (NamespaceConflictException success) {
+            assertNotNull(success.getMessage());
+        }
+        
+    }
 
     
     public void testConstructor1() {
         
-        String name = "sakjdhjhd";
+        String name = "Jethro";
         Element e = new Element(name);
         
         assertEquals(name, e.getLocalName());
@@ -246,6 +292,12 @@ public class ElementTest extends XOMTestCase {
     }
     
     
+    public void testPhilip() {
+        Element element = new Element("x", "x:&");
+        assertEquals("<x xmlns=\"x:&amp;\" />", element.toXML());
+    }
+    
+    
     public void testToXMLWithXMLLangAttribute() {
         Element e = new Element("e");
         e.addAttribute(new Attribute("xml:lang",
@@ -264,6 +316,64 @@ public class ElementTest extends XOMTestCase {
         assertEquals("", e.getNamespaceURI());
         
     }
+    
+    
+   public void testInsertChildAfterEnd() {
+        
+        Element parent = new Element("p");
+        for (int i = 0; i < 3; i++) {
+            Element c = new Element("c");
+            parent.appendChild(c);
+        }
+        assertEquals(3, parent.getChildCount());
+        Element d = new Element("d");
+        
+        try {
+          parent.insertChild(d, 5);
+          fail("Inserted child too far");
+        } 
+        catch (IndexOutOfBoundsException ex) {
+            assertNotNull(ex.getMessage());
+        }
+        
+    }
+
+
+   public void testInsertChildAtEnd() {
+       
+       Element parent = new Element("p");
+       for (int i = 0; i < 3; i++) {
+           Element c = new Element("c");
+           parent.appendChild(c);
+       }
+       assertEquals(3, parent.getChildCount());
+       Element d = new Element("d");
+       
+       parent.insertChild(d, 3);
+       assertEquals(4, parent.getChildCount());
+       
+   }
+
+
+   public void testInsertChildNegative() {
+       
+       Element parent = new Element("p");
+       for (int i = 0; i < 3; i++) {
+           Element c = new Element("c");
+           parent.appendChild(c);
+       }
+       assertEquals(3, parent.getChildCount());
+       Element d = new Element("d");
+       
+       try {
+         parent.insertChild(d, -1);
+         fail("Inserted child with negative index");
+       } 
+       catch (IndexOutOfBoundsException ex) {
+           assertNotNull(ex);
+       }
+       
+   }
 
 
     public void testCantInsertDoctype() {
@@ -444,6 +554,23 @@ public class ElementTest extends XOMTestCase {
             assertNotNull(success.getMessage());
         }        
     }
+    
+    
+    public void testConflictingDefaultNamespace() {
+        
+        Element e = new Element("foo", "http://www.foo.com/");
+        try {
+            e.addNamespaceDeclaration("", "http://www.bar.com/");
+            fail("Added conflicting default namespace");
+        }
+        catch (NamespaceConflictException success) {
+            String message = success.getMessage();
+            assertTrue(message.indexOf("default namespace") > 0);
+            assertTrue(message.indexOf("http://www.foo.com") > 0);
+            assertTrue(message.indexOf("http://www.bar.com/") > 0);
+        }
+        
+    }
 
         
     public void testNamespaceMappings() {
@@ -502,8 +629,10 @@ public class ElementTest extends XOMTestCase {
             e.addNamespaceDeclaration("pre1", "http://www.blue2.com");
             fail("Added conflicting namespace");    
         }
-        catch (NamespaceConflictException success) { 
-            assertNotNull(success.getMessage());
+        catch (NamespaceConflictException success) {
+            String message = success.getMessage();
+            assertTrue(message.indexOf("http://www.blue2.com") > 0);
+            assertTrue(message.indexOf("pre1") > 0);
         }
         
         try {
@@ -798,6 +927,19 @@ public class ElementTest extends XOMTestCase {
              
     }
     
+    public void testRFC2396LegalRFC3986IllegalNamespaceURI() {
+        
+        String name = "red:green";
+        String uri = "dcp.tcp.pft://192.168.0.1:1002:3002?fec=1&crc=0'";
+        try {
+            new Element(name, uri);
+            fail("Allowed RFC 2396 legal but RFC 3986 illegal URI");
+        }
+        catch (MalformedURIException success) {
+            assertNotNull(success.getMessage());
+        }
+             
+    }
 
     public void 
       testSetNamespaceURIConflictsWithAdditionalNamespaceDeclaration() 
@@ -1013,8 +1155,7 @@ public class ElementTest extends XOMTestCase {
             fail("Added comment before start"); 
         }
         catch (IndexOutOfBoundsException success) {
-            // success   
-            assertNotNull(success.getMessage()); 
+            // success 
         }     
         
     }
@@ -1251,6 +1392,20 @@ public class ElementTest extends XOMTestCase {
     }
     
     
+    public void testCopyElementWithAttributes() {
+        
+        Element parent = new Element("parent");
+        Attribute a = new Attribute("name", "value");
+        parent.addAttribute(a);
+        Element copy = new Element(parent);
+        assertEquals(parent, copy);
+        Attribute copied = copy.getAttribute(0);
+        assertEquals(a, copied);
+        assertEquals(copy, copied.getParent());
+        
+    }
+    
+    
     public void testCopyEmptyElement() {
         
         Element parent = new Element("parent");
@@ -1287,7 +1442,6 @@ public class ElementTest extends XOMTestCase {
         e.addAttribute(a2);
         e.addAttribute(a3);
         
-        
         Element e2 = new Element("mv:child", "http://www.mauve.com");
         e.appendChild(e2);
         
@@ -1306,8 +1460,10 @@ public class ElementTest extends XOMTestCase {
           e.getNamespaceURI("blue"), 
           copy.getNamespaceURI("blue"));
         assertEquals(e.getValue(), copy.getValue());
-        assertEquals(e.getAttribute("test").getValue(), 
-          copy.getAttribute("test").getValue());
+        
+        Attribute ea = e.getAttribute("test");
+        Attribute ca = copy.getAttribute("test");
+        assertEquals(ea.getValue(), ca.getValue());
         assertEquals(e.getBaseURI(), copy.getBaseURI());
 
     }
